# -*- coding: utf-8 -*-
# pylint: disable-msg = C0103, W0201, W0232, W0302, W0612, W0613, W0703, W0704
# pylint: disable-msg = E0213
#
# Copyright 2004-2006 André Malo or his licensors, as applicable
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Testcases for svnmailer.cli

@var _tests: The test classes
@type _tests: C{list}
"""
__author__    = "André Malo"
__docformat__ = "epytext en"
__all__       = ['getSuite']

import unittest, svnmailer_test.util
_tests = []


#### Test Bases                                     ####
########################################################
class mockingTestBase(svnmailer_test.util.MockingBase):
    """ Base class for module mocking cli tests """

    def getOptparseDict(self):
        """ Returns the dict needed for svnmailer.cli initialization """
        class OptionParser:
            def __init__(this, *args, **kwargs):
                pass
            def parse_args(this, *args, **kwargs):
                pass
            def _get_prog_name(this):
                return "gorp"
            def format_help(this, formatter = None):
                if formatter is None:
                    raise RuntimeError("No formatter given")
                return formatter._do_format()
        class Formatter:
            min_width = 20
            def __init__(this, *args, **kwargs):
                this.width = kwargs.get("width")
            def _do_format(this):
                if this.width < this.min_width:
                    raise ValueError("doh!")
                return "[%d formatted help]\n" % this.width

        return {
            'OptionParser': OptionParser,
            'IndentedHelpFormatter': Formatter,
        }

    def getSubversionDict(self):
        """ Returns the dict needed for svnmailer.cli testing """
        class Container(object):
            major = 1
            minor = 2
            patch = 3
            revision = 4
            tag = "foo"
            min_1_2 = True

        return {
            'version': Container()
        }

    def getTextwrapDict(self):
        """ Returns the dict needed for svnmailer.cli testing """
        def fillfunc(text, *args, **kwargs):
            return "BROKEN\n%s" % text

        return {
            'fill': fillfunc,
        }

    def getInitialMocks(self):
        """ Returns the list initially mocked modules """
        return [
            ('textwrap', self.getTextwrapDict()),
            ('optparse', self.getOptparseDict()),
            ('svnmailer.subversion', self.getSubversionDict()),
        ]

class cliTestBase(mockingTestBase):
    """ Bases class for cli tests """

    def setUp(self):
        """ set up test environment """
        super(cliTestBase, self).setUp()

        from svnmailer import cli
        reload(cli) # make sure to initialize again
        self._cli = cli

    def tearDown(self):
        """ tear down test environment """
        del self._cli
        super(cliTestBase, self).tearDown()

        # leave a clean module around
        from svnmailer import cli
        reload(cli)


#### Test Cases                                     ####
########################################################
class importTestCase(mockingTestBase):
    """ Importing svnmailer.cli """

    def testImport(self):
        """ from svnmailer import cli """
        try:
            from svnmailer import cli
        except Exception, exc:
            self.fail("svnmailer.cli could not be imported: %s" % str(exc))

_tests.append(importTestCase)


class ErrorTestCase(cliTestBase):
    """ svnmailer.cli.Error """

    def test00super(self):
        """ Error is an Exception """
        self.failUnless(issubclass(self._cli.Error, Exception),
            "Is not an Exception, but %r" % (self._cli.Error,)
        )

_tests.append(ErrorTestCase)


class CommandlineErrorTestCase(cliTestBase):
    """ svnmailer.cli.CommandlineError """

    def test00super(self):
        """ CommandlineError is an Error """
        expected = self._cli.Error
        result = self._cli.CommandlineError

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result)
        )

_tests.append(CommandlineErrorTestCase)


class getOptionsTestCase(cliTestBase):
    """ svnmailer.cli.getOptions """

    def setUp(self):
        """ create test environment """
        super(getOptionsTestCase, self).setUp()

        class Options:
            revision = 1
            repository = '1'
            mode = 'commit',
            author = None
            propname = None
            path_encoding = "none"
            config = None

        class Parser:
            callback = lambda *a, **k: None
            callback2 = lambda *a, **k: (Options(), [])

            def __init__(this, *arg, **kw):
                this.callback(*arg, **kw)
            def parse_args(this, argv):
                return this.callback2(argv)

        self._cli.SvnmailerOptionParser = Parser
        self._options = Options


    def test00parser(self):
        """ getOptions calls SvnmailerOptionParser properly """
        expected = ["%prog <options>", True]
        result = []
        def cb(this = None, usage = None, version = None):
            result.extend([usage, version])
        self._cli.SvnmailerOptionParser.callback = cb
        self._cli.getOptions()

        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test01parse(self):
        """ getOptions calls parse_args with supplied argv """
        expected = [None, "foo", None, self, []]
        result = []
        def cb(this = None, argv = 1):
            result.append(argv)
            return (self._options(), [])
        self._cli.SvnmailerOptionParser.callback2 = cb

        self._cli.getOptions()
        for item in ("foo", None, self, []):
            self._cli.getOptions(item)

        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test02return(self):
        """ getOptions returns the options parsed """
        parsed = self._options()
        parsed._magic = self
        def cb(this = None, argv = 1):
            return (parsed, [])
        self._cli.SvnmailerOptionParser.callback2 = cb

        expected = self
        result = self._cli.getOptions()._magic

        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test03check(self):
        """ getOptions rejects non-option arguments """
        parsed = self._options()
        def cb(this = None, argv = 1):
            return (parsed, ['a'])
        self._cli.SvnmailerOptionParser.callback2 = cb

        try:
            self._cli.getOptions()
        except self._cli.CommandlineError:
            # cool
            pass
        else:
            self.fail("Doesn't raise CommandlineError")

    def test04check(self):
        """ getOptions requires a repository path """
        parsed = self._options()
        parsed.repository = None
        def cb(this = None, argv = 1):
            return (parsed, [])
        self._cli.SvnmailerOptionParser.callback2 = cb

        try:
            self._cli.getOptions()
        except self._cli.CommandlineError:
            # cool
            pass
        else:
            self.fail("Doesn't raise CommandlineError")

    def test05check(self):
        """ getOptions requires a revision number """
        tests = ("commit", "propchange")
        parsed = self._options()
        parsed.revision = None
        def cb(this = None, argv = 1):
            return (parsed, [])
        self._cli.SvnmailerOptionParser.callback2 = cb

        for test in tests:
            parsed.mode = test
            try:
                self._cli.getOptions()
            except self._cli.CommandlineError:
                # cool
                pass
            else:
                self.fail("Input %r doesn't raise CommandlineError" % (test,))

    def test06check(self):
        """ getOptions doesn't require a revision number """
        tests = ("lock", "unlock")
        parsed = self._options()
        parsed.author = "foo"
        def cb(this = None, argv = 1):
            return (parsed, [])
        self._cli.SvnmailerOptionParser.callback2 = cb

        for test in tests:
            parsed.mode = test
            parsed.revision = None
            self._cli.getOptions()
            parsed.revision = 1
            self._cli.getOptions()

    def test07check(self):
        """ getOptions requires an author """
        tests = ("propchange", "lock", "unlock")
        parsed = self._options()
        parsed.author = None
        parsed.propname = 'a'
        def cb(this = None, argv = 1):
            return (parsed, [])
        self._cli.SvnmailerOptionParser.callback2 = cb

        for test in tests:
            parsed.mode = test
            try:
                self._cli.getOptions()
            except self._cli.CommandlineError:
                # cool
                pass
            else:
                self.fail("Input %r doesn't raise CommandlineError" % (test,))

    def test08check(self):
        """ getOptions doesn't require an author """
        parsed = self._options()
        parsed.mode = "commit"
        def cb(this = None, argv = 1):
            return (parsed, [])
        self._cli.SvnmailerOptionParser.callback2 = cb

        parsed.author = None
        self._cli.getOptions()
        parsed.author = "foo"
        self._cli.getOptions()

    def test09check(self):
        """ getOptions requires a propname """
        parsed = self._options()
        parsed.mode = "propchange"
        parsed.author = 'a'
        parsed.propname = None
        def cb(this = None, argv = 1):
            return (parsed, [])
        self._cli.SvnmailerOptionParser.callback2 = cb

        try:
            self._cli.getOptions()
        except self._cli.CommandlineError:
            # cool
            pass
        else:
            self.fail("Doesn't raise CommandlineError")

    def test10check(self):
        """ getOptions doesn't require a propname """
        tests = ("commit", "lock", "unlock")
        parsed = self._options()
        parsed.author = "foo"
        def cb(this = None, argv = 1):
            return (parsed, [])
        self._cli.SvnmailerOptionParser.callback2 = cb

        for test in tests:
            parsed.mode = test
            parsed.propname = None
            self._cli.getOptions()
            parsed.propname = "foo"
            self._cli.getOptions()

    def test11transform(self):
        """ getOptions transforms the repository path """
        parsed = self._options()
        parsed.repository = "\xa4uro"
        parsed.path_encoding = "iso-8859-15"
        def cb(this = None, argv = 1):
            return (parsed, [])
        self._cli.SvnmailerOptionParser.callback2 = cb

        options = self._cli.getOptions()
        expected = u"\u20acuro"
        result = options.repository
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test12transform(self):
        """ getOptions raises CommandlineError on repository path """
        parsed = self._options()
        parsed.repository = "\xa4uro"
        parsed.path_encoding = "utf-8"
        def cb(this = None, argv = 1):
            return (parsed, [])
        self._cli.SvnmailerOptionParser.callback2 = cb

        try:
            options = self._cli.getOptions()
        except self._cli.CommandlineError:
            # fine
            pass
        else:
            self.fail("Doesn't raise CommandlineError")

    def test13transform(self):
        """ getOptions transforms the config file """
        parsed = self._options()
        parsed.config = "\xa4uro"
        parsed.path_encoding = "iso-8859-15"
        def cb(this = None, argv = 1):
            return (parsed, [])
        self._cli.SvnmailerOptionParser.callback2 = cb

        options = self._cli.getOptions()
        expected = u"\u20acuro"
        result = options.config
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test14transform(self):
        """ getOptions raises CommandlineError on config path """
        parsed = self._options()
        parsed.config = "\xa4uro"
        parsed.path_encoding = "utf-8"
        def cb(this = None, argv = 1):
            return (parsed, [])
        self._cli.SvnmailerOptionParser.callback2 = cb

        try:
            options = self._cli.getOptions()
        except self._cli.CommandlineError:
            # fine
            pass
        else:
            self.fail("Doesn't raise CommandlineError")

_tests.append(getOptionsTestCase)


class SvnmailerOptionParserTestCase(cliTestBase):
    """ svnmailer.cli.SvnmailerOptionParser """

    def test00super(self):
        """ SvnmailerOptionParser is an optparse.OptionParser """
        expected = self._mocked['optparse'].OptionParser
        result = self._cli.SvnmailerOptionParser
        self.failUnless(issubclass(result, expected),
            "Is not a %r, but %r" % (expected, result)
        )

    def test01init(self):
        """ SvnmailerOptionParser.__init__ initializes properly """
        expected = [('1', '2'), {'foo': 'bar', 'baz': 'zonk'}, "custom_init"]
        result = []
        
        # this indirection via self.__base_class is pylint-0.6.4-anti-crash
        # hack...
        self.__base_class = self._mocked['optparse'].OptionParser
        class OptionParser(self.__base_class):
            def __init__(this, *args, **kwargs):
                result.extend([args, kwargs])
        self._cli.optparse.OptionParser = OptionParser
        reload(self._cli)
        class MyParser(self._cli.SvnmailerOptionParser):
            def _initSvnmailer(this):
                result.append("custom_init")

        MyParser('1', '2', foo = "bar", baz = "zonk")
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test02parse(self):
        """ SvnmailerOptionParser.parse_args calls transforms old style """
        expected = ["foo", "bar"]
        result = []
        class Container(object):
            pass
        class OptionParser(self._mocked['optparse'].OptionParser):
            def parse_args(this, args):
                result.append(args)
                return (Container(), [])
        self._cli.optparse.OptionParser = OptionParser
        reload(self._cli)
        class MyParser(self._cli.SvnmailerOptionParser):
            def _initSvnmailer(this):
                pass
            def _transformSvnmailerOldStyle(this, args):
                result.append(args)
                return "bar"

        MyParser().parse_args("foo")
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test03parse(self):
        """ SvnmailerOptionParser.parse_args rejects empty command lines """
        class MyParser(self._cli.SvnmailerOptionParser):
            def _initSvnmailer(this):
                pass
            def _transformSvnmailerOldStyle(this, args):
                return args
        
        parser = MyParser()
        try:
            parser.parse_args([])
        except self._cli.CommandlineError:
            # fine
            pass
        else:
            self.fail("Doesn't raise CommandlineError")

    def test04parse(self):
        """ SvnmailerOptionParser.parse_args forces action to be present """
        class Container(object):
            action = "A"
        class OptionParser(self._mocked['optparse'].OptionParser):
            def parse_args(this, args):
                return (Container(), [])
        self._cli.optparse.OptionParser = OptionParser
        reload(self._cli)
        class MyParser(self._cli.SvnmailerOptionParser):
            def _initSvnmailer(this):
                pass
            def _transformSvnmailerOldStyle(this, args):
                return args
        parser = MyParser()

        self._mocked['svnmailer.subversion'].version.min_1_2 = False
        options, fixed = parser.parse_args(["foo"])
        expected = None
        result = options.action
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

        self._mocked['svnmailer.subversion'].version.min_1_2 = True
        options, fixed = parser.parse_args(["foo"])
        expected = "A"
        result = options.action
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test05error(self):
        """ SvnmailerOptionParser.error raises CommandlineError """
        class MyParser(self._cli.SvnmailerOptionParser):
            def _initSvnmailer(this):
                pass

        parser = MyParser()
        try:
            parser.error("foo")
        except self._cli.CommandlineError, exc:
            # yup
            expected = "foo"
            result = str(exc)
            self.failUnlessEqual(expected, result,
                "Expected %r, got %r" % (expected, result)
            )
        else:
            self.fail("Doesn't raise CommandlineError")

    def test06getversion(self):
        """ SvnmailerOptionParser.get_version returns proper infos """
        import svnmailer

        class MyParser(self._cli.SvnmailerOptionParser):
            def _initSvnmailer(this):
                pass

        parser = MyParser()
        expected = "svnmailer-3.14159265\nwith svn 1.2.3foo"
        old_version = svnmailer.version
        try:
            svnmailer.version = "3.14159265"
            result = parser.get_version()
        finally:
            svnmailer.version = old_version

        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )
        
    def test07formathelp(self):
        """ SvnmailerOptionParser.format_help adds old style infos """
        class MyParser(self._cli.SvnmailerOptionParser):
            def _initSvnmailer(this):
                pass
            def _getTerminalWidth(this):
                return 132

        parser = MyParser()

        self._mocked["svnmailer.subversion"].version.min_1_2 = False
        expected = "[131 formatted help]\n\nBROKEN\nAlternatively you can " \
            "use the old style compatibility command lines (options " \
            "described above don't apply then):\nBROKEN\n\n" \
            "BROKEN\ngorp commit <repos> <revision> [<config>]\n" \
            "BROKEN\ngorp propchange <repos> <revision> <author> <propname> " \
            "[<config>]\n"

        result = parser.format_help()
        self.failUnlessEqual(expected, result,
            "1st: Expected %r, got %r" % (expected, result)
        )

        self._mocked["svnmailer.subversion"].version.min_1_2 = True
        expected = "[131 formatted help]\n\nBROKEN\nAlternatively you can " \
            "use the old style compatibility command lines (options " \
            "described above don't apply then):\nBROKEN\n\n" \
            "BROKEN\ngorp commit <repos> <revision> [<config>]\n" \
            "BROKEN\ngorp propchange <repos> <revision> <author> <propname> " \
            "[<config>]\n" \
            "BROKEN\n\n" \
            "BROKEN\nsvn 1.2 and later:\n" \
            "BROKEN\ngorp propchange2 <repos> <revision> <author> " \
            "<propname> <action> [<config>]\n" \
            "BROKEN\ngorp lock <repos> <author> [<config>]\n" \
            "BROKEN\ngorp unlock <repos> <author> [<config>]\n"

        result = parser.format_help()
        self.failUnlessEqual(expected, result,
            "2nd: Expected %r, got %r" % (expected, result)
        )

    def test08formathelp(self):
        """ SvnmailerOptionsParser.format_help deals with small terminals """
        class MyParser(self._cli.SvnmailerOptionParser):
            _twidth = 20
            def _initSvnmailer(this):
                pass
            def _getTerminalWidth(this):
                return this._twidth
        parser = MyParser()

        expected = "[79 formatted help]"
        result = parser.format_help()[:len(expected)]
        self.failUnlessEqual(expected, result,
            "1st: Expected %r, got %r" % (expected, result)
        )

        self._mocked["optparse"].IndentedHelpFormatter.min_width = 85
        expected = "[89 formatted help]"
        result = parser.format_help()[:len(expected)]
        self.failUnlessEqual(expected, result,
            "2nd: Expected %r, got %r" % (expected, result)
        )
 
    def test09getTerminalWidth(self):
        """ SvnmailerOptionParser._getTerminalWidth does proper syscalls """
        import sys
        result = []
        mole = [True, None]

        def ioctlfunc(*args):
            result.append(args)
            if mole[0] and args[0] == sys.stdout.fileno():
                raise IOError(10, "foo")
            return (1, 2, 3, 4)
        def packfunc(*args):
            result.append(args)
            return "\1\2\3\4"
        def unpackfunc(*args):
            result.append(args)
            if mole[1]:
                raise mole[1]()
            return args[1]

        self.addMock("errno", {"EINVAL": 10})
        self.addMock("fcntl", {"ioctl": ioctlfunc})
        self.addMock("struct", {"pack": packfunc, "unpack": unpackfunc})
        self.addMock("termios", {"TIOCGWINSZ": 31415})

        class MyParser(self._cli.SvnmailerOptionParser):
            def _initSvnmailer(this):
                pass
        parser = MyParser()

        expected = 2
        width = parser._getTerminalWidth()
        self.failUnlessEqual(expected, width,
            "1st: Expected %r, got %r" % (expected, width)
        )

        expected = [
            ('4H', 0, 0, 0, 0),
            (sys.stdout.fileno(), 31415, '\x01\x02\x03\x04'),
            ('4H', 0, 0, 0, 0),
            (sys.stdin.fileno(), 31415, '\x01\x02\x03\x04'),
            ('4H', (1, 2, 3, 4)),
        ]
        self.failUnlessEqual(expected, result,
            "2nd: Expected %r, got %r" % (expected, result)
        )

        mole[0] = False
        result[:] = []
        expected = [
            ('4H', 0, 0, 0, 0),
            (sys.stdout.fileno(), 31415, '\x01\x02\x03\x04'),
            ('4H', (1, 2, 3, 4)),
        ]
        parser._getTerminalWidth()
        self.failUnlessEqual(expected, result,
            "3rd: Expected %r, got %r" % (expected, result)
        )

        mole[1] = SystemExit
        try:
            parser._getTerminalWidth()
            self.fail("Doesn't raise SystemExit")
        except SystemExit:
            # fine
            pass

        mole[1] = KeyboardInterrupt
        try:
            parser._getTerminalWidth()
            self.fail("Doesn't raise KeyboardInterrupt")
        except KeyboardInterrupt:
            # fine
            pass

        self._mocked["errno"].EINVAL = 11
        mole[0] = True
        expected = None
        result = parser._getTerminalWidth()
        self.failUnlessEqual(expected, result,
            "4th: Expected %r, got %r" % (expected, result)
        )

    def test10customInit(self):
        """ SvnmailerOptionParser._initSvnmailer adds the groups properly """
        expected = ["common", "behaviour", "supplemental"]
        result = []
        class MyParser(self._cli.SvnmailerOptionParser):
            def _addSvnmailerCommonOptions(this):
                result.append("common")
            def _addSvnmailerBehaviourOptions(this):
                result.append("behaviour")
            def _addSvnmailerSupplementalOptions(this):
                result.append("supplemental")

        MyParser()
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test11commonOptions(self):
        """ SvnmailerOptionParser._addSvnmailerCommonOptions adds """
        result = []
        class OptionGroup:
            def __init__(this, *args):
                result.append(args)
            def add_option(this, *args, **kwargs):
                result.extend([args, kwargs])
        self._cli.optparse.OptionGroup = OptionGroup

        class MyParser(self._cli.SvnmailerOptionParser):
            def _initSvnmailer(this):
                pass
            def add_option_group(this, group):
                result.append(self)

        parser = MyParser()
        expected = [
            (parser, 'COMMON PARAMETERS'),
            ('--debug',),
            {'action': 'store_true', 'default': False,
             'help': 'Run in debug mode (means basically that all messages '
                     'are sent to STDOUT)'},
            ('-d', '--repository'), {'help': 'The repository directory'},
            ('-f', '--config'), {'help': 'The configuration file'},
            ('-e', '--path-encoding'),
            {'help': 'Specifies the character encoding to be used for '
                     'filenames. By default the encoding is tried to be '
                     'determined automatically depending on the locale.'},
            self
        ]
        parser._addSvnmailerCommonOptions()

        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test12behaviourOptions(self):
        """ SvnmailerOptionParser._addSvnmailerBehaviourOptions adds """
        result = []
        class OptionGroup:
            def __init__(this, *args, **kwargs):
                result.append(args)
                result.append(kwargs)
            def add_option(this, *args, **kwargs):
                result.extend([args, kwargs])
        self._cli.optparse.OptionGroup = OptionGroup

        class MyParser(self._cli.SvnmailerOptionParser):
            def _initSvnmailer(this):
                pass
            def add_option_group(this, group):
                result.append(self)
        parser = MyParser()

        self._mocked["svnmailer.subversion"].version.min_1_2 = False
        expected = [
            (parser, 'BEHAVIOUR OPTIONS'),
            {'description': 'The behaviour options are mutually exclusive, '
                            'i.e. the last one wins.'},
            ('-c', '--commit'),
            {'dest': 'mode', 'action': 'store_const', 'const': 'commit',
             'default': 'commit',
             'help': 'This is a regular commit of versioned data '
                     '(post-commit hook). This is default.'},
            ('-p', '--propchange'),
            {'dest': 'mode', 'action': 'store_const', 'const': 'propchange',
             'help': 'This is a modification of unversioned properties '
                     '(post-revprop-change hook)'},
            self
        ]
        parser._addSvnmailerBehaviourOptions()
        self.failUnlessEqual(expected, result,
            "1st: Expected %r, got %r" % (expected, result)
        )

        self._mocked["svnmailer.subversion"].version.min_1_2 = True
        expected = [
            (parser, 'BEHAVIOUR OPTIONS'),
            {'description': 'The behaviour options are mutually exclusive, '
                            'i.e. the last one wins.'},
            ('-c', '--commit'),
            {'dest': 'mode', 'action': 'store_const', 'const': 'commit',
             'default': 'commit',
             'help': 'This is a regular commit of versioned data '
                     '(post-commit hook). This is default.'},
            ('-p', '--propchange'),
            {'dest': 'mode', 'action': 'store_const', 'const': 'propchange',
             'help': 'This is a modification of unversioned properties '
                     '(post-revprop-change hook)'},
            ('-l', '--lock'),
            {'dest': 'mode', 'action': 'store_const', 'const': 'lock',
             'help': '(svn 1.2 and later) This is a locking call '
                     '(post-lock hook)'},
            ('-u', '--unlock'),
            {'dest': 'mode', 'action': 'store_const', 'const': 'unlock',
             'help': '(svn 1.2 and later) This is a unlocking call '
                     '(post-unlock hook)'},
            self
        ]
        result[:] = [] # cleanup
        parser._addSvnmailerBehaviourOptions()
        self.failUnlessEqual(expected, result,
            "2nd: Expected %r, got %r" % (expected, result)
        )

    def test13supplementalOptions(self):
        """ SvnmailerOptionParser._addSvnmailerSupplmentalOptions adds """
        result = []
        class OptionGroup:
            def __init__(this, *args):
                result.append(args)
            def add_option(this, *args, **kwargs):
                result.extend([args, kwargs])
        self._cli.optparse.OptionGroup = OptionGroup

        class MyParser(self._cli.SvnmailerOptionParser):
            def _initSvnmailer(this):
                pass
            def add_option_group(this, group):
                result.append(self)
        parser = MyParser()

        self._mocked["svnmailer.subversion"].version.min_1_2 = False
        expected = [
            (parser, 'SUPPLEMENTAL PARAMETERS'),
            ('-r', '--revision'),
            {'action': 'store', 'type': 'int',
             'help': 'The modified/committed revision number'},
            ('-a', '--author'), {'help': 'The author of the modification'},
            ('-n', '--propname'),
            {'help': 'The name of the modified property'},
            self
        ]
        parser._addSvnmailerSupplementalOptions()
        self.failUnlessEqual(expected, result,
            "1st: Expected %r, got %r" % (expected, result)
        )

        self._mocked["svnmailer.subversion"].version.min_1_2 = True
        expected = [
            (parser, 'SUPPLEMENTAL PARAMETERS'),
            ('-r', '--revision'),
            {'action': 'store', 'type': 'int',
             'help': 'The modified/committed revision number'},
            ('-a', '--author'), {'help': 'The author of the modification'},
            ('-n', '--propname'),
            {'help': 'The name of the modified property'},
            ('-o', '--action'),
            {'help': '(svn 1.2 and later) The property change action'},
            self
        ]
        result[:] = []
        parser._addSvnmailerSupplementalOptions()
        self.failUnlessEqual(expected, result,
            "2nd: Expected %r, got %r" % (expected, result)
        )

    def test14transform(self):
        """ SvnmailerOptionParser._transformSvnmailerOldStyle passes argv """
        class MyParser(self._cli.SvnmailerOptionParser):
            def _initSvnmailer(this):
                pass

        parser = MyParser()
        test = ['bar', 'baz']

        expected = ['bar', 'baz']
        result = parser._transformSvnmailerOldStyle(test)
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test15transform(self):
        """ SvnmailerOptionParser._transformSvnmailerOldStyle uses sys.argv """
        import sys
        class MyParser(self._cli.SvnmailerOptionParser):
            def _initSvnmailer(this):
                pass

        parser = MyParser()
        old_argv = sys.argv
        try:
            sys.argv = ['foo', 'bar', 'baz']
            expected = ['bar', 'baz']
            result = parser._transformSvnmailerOldStyle(None)
        finally:
            sys.argv = old_argv

        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test16transform(self):
        """ SvnmailerOptionParser transforms commit subcommands """
        class MyParser(self._cli.SvnmailerOptionParser):
            def _initSvnmailer(this):
                pass

        tests = (
            (['--commit', '--repository', 'repos', '--revision', '1'],
             ['commit', 'repos', '1']),
            (['--commit', '--repository', 'repos2', '--revision', '3',
              '--config', 'conf'],
             ['commit', 'repos2', '3', 'conf']),
        )
        parser = MyParser()
        for expected, test in tests:
            result = parser._transformSvnmailerOldStyle(test)
            self.failUnlessEqual(expected, result,
                "Expected %r, got %r" % (expected, result)
            )

    def test17transform(self):
        """ SvnmailerOptionParser passes wrong commit subcommands """
        class MyParser(self._cli.SvnmailerOptionParser):
            def _initSvnmailer(this):
                pass

        tests = (
            (['commit', 'repos'], ['commit', 'repos']),
            (['commit', 'repos2', '3', 'conf', 'foo'],
             ['commit', 'repos2', '3', 'conf', 'foo']),
        )
        parser = MyParser()
        for expected, test in tests:
            result = parser._transformSvnmailerOldStyle(test)
            self.failUnlessEqual(expected, result,
                "Expected %r, got %r" % (expected, result)
            )

    def test18transform(self):
        """ SvnmailerOptionParser transforms propchange subcommands """
        class MyParser(self._cli.SvnmailerOptionParser):
            def _initSvnmailer(this):
                pass

        tests = (
            (['--propchange', '--repository', 'repos', '--revision', '1',
              '--author', 'nd', '--propname', 'svn:log'],
             ['propchange', 'repos', '1', 'nd', 'svn:log']),
            (['--propchange', '--repository', 'repos2', '--revision', '3',
              '--author', 'malo', '--propname', 'svn:author',
              '--config', 'conf'],
             ['propchange', 'repos2', '3', 'malo', 'svn:author', 'conf']),
        )
        parser = MyParser()
        for expected, test in tests:
            result = parser._transformSvnmailerOldStyle(test)
            self.failUnlessEqual(expected, result,
                "Expected %r, got %r" % (expected, result)
            )

    def test19transform(self):
        """ SvnmailerOptionParser passes wrong propchange subcommands """
        class MyParser(self._cli.SvnmailerOptionParser):
            def _initSvnmailer(this):
                pass

        tests = (
            (['propchange', 'repos', '1', 'nd'],
             ['propchange', 'repos', '1', 'nd']),
            (['propchange', 'repos2', '3', 'nd', 'svn:log', 'conf', 'foo'],
             ['propchange', 'repos2', '3', 'nd', 'svn:log', 'conf', 'foo']),
        )
        parser = MyParser()
        for expected, test in tests:
            result = parser._transformSvnmailerOldStyle(test)
            self.failUnlessEqual(expected, result,
                "Expected %r, got %r" % (expected, result)
            )

    def test20transform(self):
        """ SvnmailerOptionParser transforms propchange2 subcommands """
        class MyParser(self._cli.SvnmailerOptionParser):
            def _initSvnmailer(this):
                pass

        tests = (
            (['--propchange', '--repository', 'repos', '--revision', '1',
              '--author', 'nd', '--propname', 'svn:log', '--action', 'A'],
             ['propchange2', 'repos', '1', 'nd', 'svn:log', 'A']),
            (['--propchange', '--repository', 'repos2', '--revision', '3',
              '--author', 'malo', '--propname', 'svn:author', '--action', 'M',
              '--config', 'conf'],
             ['propchange2', 'repos2', '3', 'malo', 'svn:author', 'M', 'conf']),
        )
        parser = MyParser()
        for expected, test in tests:
            self._mocked["svnmailer.subversion"].version.min_1_2 = True
            result = parser._transformSvnmailerOldStyle(test)
            self.failUnlessEqual(expected, result,
                "Expected %r, got %r" % (expected, result)
            )

            self._mocked["svnmailer.subversion"].version.min_1_2 = False
            result = parser._transformSvnmailerOldStyle(test)
            self.failIfEqual(expected, result,
                "Expected %r, got %r" % (expected, result)
            )

    def test21transform(self):
        """ SvnmailerOptionParser passes wrong propchange2 subcommands """
        class MyParser(self._cli.SvnmailerOptionParser):
            def _initSvnmailer(this):
                pass

        tests = (
            (['propchange2', 'repos', '1', 'nd', 'svn:author'],
             ['propchange2', 'repos', '1', 'nd', 'svn:author']),
            (['propchange2', 'repos2', '3', 'nd', 'svn:log', 'M', 'conf',
              'foo'],
             ['propchange2', 'repos2', '3', 'nd', 'svn:log', 'M', 'conf',
              'foo']),
        )
        parser = MyParser()
        for expected, test in tests:
            for svn12 in (True, False):
                self._mocked["svnmailer.subversion"].version.min_1_2 = svn12
                result = parser._transformSvnmailerOldStyle(test)
                self.failUnlessEqual(expected, result,
                    "Expected %r, got %r" % (expected, result)
                )

    def test22transform(self):
        """ SvnmailerOptionParser transforms lock subcommands """
        class MyParser(self._cli.SvnmailerOptionParser):
            def _initSvnmailer(this):
                pass

        tests = (
            (['--lock', '--repository', 'repos', '--author', 'nd'],
             ['lock', 'repos', 'nd']),
            (['--lock', '--repository', 'repos2', '--author', 'malo',
              '--config', 'conf'],
             ['lock', 'repos2', 'malo', 'conf']),
        )
        parser = MyParser()
        for expected, test in tests:
            self._mocked["svnmailer.subversion"].version.min_1_2 = True
            result = parser._transformSvnmailerOldStyle(test)
            self.failUnlessEqual(expected, result,
                "Expected %r, got %r" % (expected, result)
            )

            self._mocked["svnmailer.subversion"].version.min_1_2 = False
            result = parser._transformSvnmailerOldStyle(test)
            self.failIfEqual(expected, result,
                "Expected %r, got %r" % (expected, result)
            )

    def test23transform(self):
        """ SvnmailerOptionParser passes wrong lock subcommands """
        class MyParser(self._cli.SvnmailerOptionParser):
            def _initSvnmailer(this):
                pass

        tests = (
            (['lock', 'repos'],
             ['lock', 'repos']),
            (['lock', 'repos2', 'nd', 'conf', 'foo'],
             ['lock', 'repos2', 'nd', 'conf', 'foo']),
        )
        parser = MyParser()
        for expected, test in tests:
            for svn12 in (True, False):
                self._mocked["svnmailer.subversion"].version.min_1_2 = svn12
                result = parser._transformSvnmailerOldStyle(test)
                self.failUnlessEqual(expected, result,
                    "Expected %r, got %r" % (expected, result)
                )

    def test24transform(self):
        """ SvnmailerOptionParser transforms unlock subcommands """
        class MyParser(self._cli.SvnmailerOptionParser):
            def _initSvnmailer(this):
                pass

        tests = (
            (['--unlock', '--repository', 'repos', '--author', 'nd'],
             ['unlock', 'repos', 'nd']),
            (['--unlock', '--repository', 'repos2', '--author', 'malo',
              '--config', 'conf'],
             ['unlock', 'repos2', 'malo', 'conf']),
        )
        parser = MyParser()
        for expected, test in tests:
            self._mocked["svnmailer.subversion"].version.min_1_2 = True
            result = parser._transformSvnmailerOldStyle(test)
            self.failUnlessEqual(expected, result,
                "Expected %r, got %r" % (expected, result)
            )

            self._mocked["svnmailer.subversion"].version.min_1_2 = False
            result = parser._transformSvnmailerOldStyle(test)
            self.failIfEqual(expected, result,
                "Expected %r, got %r" % (expected, result)
            )

    def test25transform(self):
        """ SvnmailerOptionParser passes wrong unlock subcommands """
        class MyParser(self._cli.SvnmailerOptionParser):
            def _initSvnmailer(this):
                pass

        tests = (
            (['unlock', 'repos'],
             ['unlock', 'repos']),
            (['unlock', 'repos2', 'nd', 'conf', 'foo'],
             ['unlock', 'repos2', 'nd', 'conf', 'foo']),
        )
        parser = MyParser()
        for expected, test in tests:
            for svn12 in (True, False):
                self._mocked["svnmailer.subversion"].version.min_1_2 = svn12
                result = parser._transformSvnmailerOldStyle(test)
                self.failUnlessEqual(expected, result,
                    "Expected %r, got %r" % (expected, result)
                )

_tests.append(SvnmailerOptionParserTestCase)


#### Glue code                                      ####
########################################################
__all__.extend(_tests)

def getSuite():
    """ Returns the test suite for this module """
    return unittest.TestSuite([
        unittest.makeSuite(test) for test in _tests
    ])

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity = 2).run(getSuite())
