# -*- coding: utf-8 -*-
# pylint: disable-msg = C0103, R0904, W0221, W0232, W0302, W0612, W0613, W0703
# pylint: disable-msg = W0706, E0213
#
# Copyright 2004-2006 André Malo or his licensors, as applicable
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Testcases for svnmailer.config

@var _tests: The test classes
@type _tests: C{list}
"""
__author__    = "André Malo"
__docformat__ = "epytext en"
__all__       = ['getSuite']

import unittest, svnmailer_test.util
_tests = []


#### Test Bases                                     ####
########################################################
class mockingTestBase(svnmailer_test.util.MockingBase):
    """ Base class for module mocking config tests """

    def getConfigParserDict(self):
        """ Returns the dict needed for ConfigParser initialization """
        class ConfigParser:
            pass

        return {'ConfigParser': ConfigParser}

    def getSettingsDict(self):
        """ Returns the dict needed for svnmailer.settings initialization """
        class Settings(object):
            pass

        return {'Settings': Settings}

    def getUtilDict(self):
        """ Returns the dict needed for svnmailer.util initialization """
        return {}

    def getInitialMocks(self):
        """ Returns the list initially mocked modules """
        return [
            ('ConfigParser', self.getConfigParserDict()),
            ('svnmailer.settings', self.getSettingsDict()),
            ('svnmailer.util', self.getUtilDict()),
        ]

class configTestBase(mockingTestBase):
    """ Bases class for config tests """

    def setUp(self):
        """ set up test environment """
        super(configTestBase, self).setUp()

        from svnmailer import config
        reload(config) # make sure to initialize again
        self._config = config

    def tearDown(self):
        """ tear down test environment """
        del self._config
        super(configTestBase, self).tearDown()

        # leave a clean module around
        from svnmailer import config
        reload(config)

class errorTestBase(configTestBase):
    """ Base class for Exception checks """
    _super_class = None
    _test_class = None

    def __init__(self, methodName = 'runTest'):
        self.methodName = methodName
        super(errorTestBase, self).__init__(methodName)
        
    def shortDescription(self):
        if self.methodName == 'test00super':
            return "%s is a %s" % (self._test_class, self._super_class)
        elif self.methodName == 'test01public':
            return "%s is public" % (self._test_class)
        else:
            raise RuntimeError("unknown method %s" % self.methodName)

    def test00super(self):
        expected = getattr(self._config, self._super_class)
        result = getattr(self._config, self._test_class)

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result)
        )

    def test01public(self):
        self.failUnless(self._test_class in self._config.__all__,
            "%r is not noted in __all__" % (self._test_class,)
        )


#### Test Cases                                     ####
########################################################
class importTestCase(unittest.TestCase):
    """ Importing svnmailer.config """

    def testImport(self):
        """ from svnmailer import config """
        try:
            from svnmailer import config
        except Exception, exc:
            self.fail("svnmailer.config could not be imported: %s" % str(exc))

_tests.append(importTestCase)


class ErrorTestCase(configTestBase):
    """ svnmailer.config.Error """

    def test00super(self):
        """ Error is an Exception """
        self.failUnless(issubclass(self._config.Error, Exception),
            "Is not an Exception, but %r" % (self._config.Error,)
        )

_tests.append(ErrorTestCase)


class ConfigNotFoundErrorTestCase(errorTestBase):
    """ svnmailer.config.ConfigNotFoundError """
    _super_class = "Error"
    _test_class = "ConfigNotFoundError"

_tests.append(ConfigNotFoundErrorTestCase)


class ConfigMissingErrorTestCase(errorTestBase):
    """ svnmailer.config.ConfigMissingError """
    _super_class = "ConfigNotFoundError"
    _test_class = "ConfigMissingError"

_tests.append(ConfigMissingErrorTestCase)


class ConfigInvalidErrorTestCase(errorTestBase):
    """ svnmailer.config.ConfigInvalidError """
    _super_class = "Error"
    _test_class = "ConfigInvalidError"

_tests.append(ConfigInvalidErrorTestCase)


class ConfigMappingSectionNotFoundErrorTestCase(errorTestBase):
    """ svnmailer.config.ConfigMappingSectionNotFoundError """
    _super_class = "ConfigInvalidError"
    _test_class = "ConfigMappingSectionNotFoundError"

_tests.append(ConfigMappingSectionNotFoundErrorTestCase)


class ConfigMappingSpecInvalidErrorTestCase(errorTestBase):
    """ svnmailer.config.ConfigMappingSpecInvalidError """
    _super_class = "ConfigInvalidError"
    _test_class = "ConfigMappingSpecInvalidError"

_tests.append(ConfigMappingSpecInvalidErrorTestCase)


class ConfigSectionNotFoundErrorTestCase(errorTestBase):
    """ svnmailer.config.ConfigSectionNotFoundError """
    _super_class = "ConfigInvalidError"
    _test_class = "ConfigSectionNotFoundError"

_tests.append(ConfigSectionNotFoundErrorTestCase)


class ConfigOptionUnknownErrorTestCase(errorTestBase):
    """ svnmailer.config.ConfigOptionUnknownError """
    _super_class = "ConfigInvalidError"
    _test_class = "ConfigOptionUnknownError"

_tests.append(ConfigOptionUnknownErrorTestCase)


class ConfigFileSettingsTestCase(configTestBase):
    """ svnmailer.config.ConfigFileSettings """

    def shortDescription(self):
        desc = super(ConfigFileSettingsTestCase, self).shortDescription()
        if desc[:1] == '.':
            desc = "ConfigFileSettings%s" % desc
        return desc

    def test00super(self):
        """ ConfigFileSettings is a svnmailer.settings.Settings """
        expected = self._mocked['svnmailer.settings'].Settings
        result = self._config.ConfigFileSettings

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result)
        )

    def test01init(self):
        """ .init calls _init properly """
        result = []
        class MySettings(self._config.ConfigFileSettings):
            def _init(this, *args, **kwargs):
                result.extend([args, kwargs])

        inst = MySettings()
        expected = [('foo', 'bar'), {'baz': 'zonk'}]
        inst.init('foo', 'bar', baz = 'zonk')

        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test02init(self):
        """ .init wraps certain Exceptions """
        class MyError(Exception):
            pass
        class MySettings(self._config.ConfigFileSettings):
            def _init(this, exc, text):
                raise exc(text)

        self._mocked['ConfigParser'].Error = MyError
        tests = (ValueError, TypeError, UnicodeError, MyError)
        inst = MySettings()

        for test in tests:
            try:
                inst.init(test, "Exc: %s - some text" % test.__name__)
            except self._config.ConfigInvalidError, exc:
                expected = "Exc: %s - some text" % test.__name__
                result = str(exc)
                self.failUnlessEqual(expected, result,
                    "Expected %r, got %r" % (expected, result)
                )
            else:
                self.fail("Doesn't raise ConfigInvalidError")

    def test03init(self):
        """ .init passes certain Exceptions """
        class MyError(Exception):
            pass
        class MySettings(self._config.ConfigFileSettings):
            def _init(this, exc, text):
                raise exc(text)

        self._mocked['ConfigParser'].Error = MyError
        tests = (
            self._config.ConfigMissingError,
            self._config.ConfigNotFoundError,
            self._config.ConfigSectionNotFoundError,
            self._config.ConfigOptionUnknownError,
            self._config.ConfigMappingSpecInvalidError,
            self._config.ConfigMappingSectionNotFoundError,
        )
        inst = MySettings()

        for test in tests:
            try:
                inst.init(test, "Exc: %s - some text" % test.__name__)
            except test, exc:
                expected = "Exc: %s - some text" % test.__name__
                result = str(exc)
                self.failUnlessEqual(expected, result,
                    "Expected %r, got %r" % (expected, result)
                )
            else:
                self.fail("Doesn't raise %s", test.__name__)

    def test04init(self):
        """ ._init calls the sub init methods properly """
        expected = ["runtime", self, "config", "general", "groups"]
        result = []

        class MySettings(self._config.ConfigFileSettings):
            def _initRuntime(this, options):
                result.extend(["runtime", options])
            def _loadConfig(this):
                result.append("config")
            def _initGeneral(this):
                result.append("general")
            def _initGroups(this):
                result.append("groups")

        inst = MySettings()
        inst._init(self)
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def _initGroupSettings(self):
        class MySettings(self._config.ConfigFileSettings):
            def __init__(this, config, defaults):
                this._config = Config(*config)
                this._test_defaults = Container(**defaults)
                this.groups = []
            def _getDefaultGroupDict(this, container):
                return container._dict_
            def _getGroupDefaults(this):
                return this._test_defaults
            def _passConfig(this, container, section):
                container._dict_.update(this._config.section(section))
            def getGroupContainer(this, **kwargs):
                return Container(**kwargs)
        return MySettings

    def test05initGroups(self):
        """ ._initGroups falls back to [default] """
        MySettings = self._initGroupSettings()

        inst = MySettings([], {'tester': self})
        inst._initGroups()
        expected = [{'tester': self}]
        result = [group._dict_ for group in inst.groups]
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test06initGroups(self):
        """ ._initGroups passes all sections """
        MySettings = self._initGroupSettings()

        inst = MySettings(
            [('foo', {'opt1': 'val1'}), ('bar', {'opt2': 'val2', 'tester': 3})],
            {'tester': self}
        )
        inst._initGroups()
        expected = [
            {'_name': 'foo', 'opt1': 'val1', 'tester': self},
            {'_name': 'bar', 'opt2': 'val2', 'tester': 3},
        ]
        result = [group._dict_ for group in inst.groups]
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test07getDefaultGroupDict(self):
        """ ._getDefaultGroupDict returns the proper dict """
        inst = self._config.ConfigFileSettings()
        container = Container(foo = 1, for_repos = 2, for_paths = 3)

        expected = {'foo': 1,
            'for_repos': 2, '_def_for_repos': 2,
            'for_paths': 3, '_def_for_paths': 3}
        result = inst._getDefaultGroupDict(container)
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test08getGroupDefaults(self):
        """ ._getGroupDefaults inits a default container """
        class MySettings(self._config.ConfigFileSettings):
            general = Container(
                diff_command = "some command",
                cia_rpc_server = "some rpc server",
            )
            _config = Config()
            def getDefaultGroupContainer(this, **kwargs):
                return Container(**kwargs)
            def _passConfig(this, container, section):
                pass

        inst = MySettings()
        expected = {
            'diff_command': "some command",
            'cia_rpc_server': "some rpc server",
            '_name': "defaults"
        }
        result = inst._getGroupDefaults()._dict_
        self.failUnlessEqual(expected, result,
            "1st: Expected %r, got %r" % (expected, result)
        )

        expected = ["defaults"]
        result = inst._config.sections_removed
        self.failUnlessEqual(expected, result,
            "2nd: Expected %r, got %r" % (expected, result)
        )
        
    def test09getGroupDefaults(self):
        """ ._getGroupDefaults passes [defaults] """
        class MySettings(self._config.ConfigFileSettings):
            general = Container(
                diff_command = "some command",
                cia_rpc_server = "some rpc server",
            )
            _config = Config(("defaults", {"foo": self, "bar": "baz"}))
            def getDefaultGroupContainer(this, **kwargs):
                return Container(**kwargs)
            def _passConfig(this, container, section):
                container._dict_.update(this._config.section(section))

        inst = MySettings()
        expected = {
            'diff_command': "some command", '_name': "defaults",
            'cia_rpc_server': "some rpc server",
            'foo': self, 'bar': "baz"
        }
        result = inst._getGroupDefaults()._dict_
        self.failUnlessEqual(expected, result,
            "1st: Expected %r, got %r" % (expected, result)
        )

        expected = ["defaults"]
        result = inst._config.sections_removed
        self.failUnlessEqual(expected, result,
            "2nd: Expected %r, got %r" % (expected, result)
        )

    def test10getGroupDefaults(self):
        """ ._getGroupDefaults ignores missing [defaults] """
        class MySettings(self._config.ConfigFileSettings):
            general = Container(
                diff_command = "some command",
                cia_rpc_server = "some rpc server"
            )
            _config = Config()
            def getDefaultGroupContainer(this, **kwargs):
                return Container(**kwargs)
            def _passConfig(this, container, section):
                raise self._config.ConfigSectionNotFoundError()

        inst = MySettings()
        expected = {
            'diff_command': "some command",
            'cia_rpc_server': "some rpc server",
            '_name': "defaults"
        }
        result = inst._getGroupDefaults()._dict_
        self.failUnlessEqual(expected, result,
            "1st: Expected %r, got %r" % (expected, result)
        )

        expected = []
        result = inst._config.sections_removed
        self.failUnlessEqual(expected, result,
            "2nd: Expected %r, got %r" % (expected, result)
        )

    def test11initGeneral(self):
        """ ._initGeneral inits a general settings container """
        class MySettings(self._config.ConfigFileSettings):
            _config = Config(('general', {'foo': self, 'bar': "baz"}))
            def getGeneralContainer(this, **kwargs):
                return Container(**kwargs)
            def _passConfig(this, container, section):
                container._dict_.update(this._config.section(section))

        inst = MySettings()
        inst._initGeneral()
        expected = {'foo': self, 'bar': "baz"}
        result = inst.general._dict_
        self.failUnlessEqual(expected, result,
            "1st: Expected %r, got %r" % (expected, result)
        )

        expected = ["general"]
        result = inst._config.sections_removed
        self.failUnlessEqual(expected, result,
            "2nd: Expected %r, got %r" % (expected, result)
        )

    def test12initGeneral(self):
        """ ._initGeneral passes ConfigSectionNotFoundError """
        class MySettings(self._config.ConfigFileSettings):
            def getGeneralContainer(this, **kwargs):
                return Container(**kwargs)
            def _passConfig(this, container, section):
                raise self._config.ConfigSectionNotFoundError()

        inst = MySettings()
        try:
            inst._initGeneral()
        except self._config.ConfigSectionNotFoundError:
            # fine.
            pass
        else:
            self.fail("Doesn't raise ConfigSectionNotFoundError")

    def test13initRuntime(self):
        """ ._initRuntime inits a runtime settings container """
        def mockedfunc(*args):
            return args
        self._mocked['svnmailer.util'].getSuitableFileName = mockedfunc

        class MySettings(self._config.ConfigFileSettings):
            def getRuntimeContainer(this, **kwargs):
                return Container(**kwargs)

        options = Container(
            repository = "foo", revision = 10, debug = True, config = "conf",
            mode = "propchange", author = None, propname = "x", action = "+",
            path_encoding = "utf-8"
        )
        inst = MySettings()
        inst._initRuntime(options)
        expected = {
            'repository': "foo", 'revision': 10, 'debug': True,
            'config': "conf", 'mode': "propchange", 'author': None,
            'propname': "x", 'action': "+", 'path_encoding': "utf-8",
        }
        result = inst.runtime._dict_
        self.failUnlessEqual(expected, result,
            "1st: Expected %r, got %r" % (expected, result)
        )

        expected = "utf-8"
        result = inst._fcharset_
        self.failUnlessEqual(expected, result,
            "2st: Expected %r, got %r" % (expected, result)
        )

    def test14passConfig(self):
        """ ._passConfig passes all section options to the container """
        class MySettings(self._config.ConfigFileSettings):
            _config = Config(("foo", {"a": 1, "b": self}))
        inst = MySettings()
        container = Container()
        inst._passConfig(container, "foo")
        expected = {"a": 1, "b": self}
        result = container._dict_
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test15passConfig(self):
        """ ._passConfig raises exception on _ options """
        class NoSectionError(Exception):
            pass
        self._mocked["ConfigParser"].NoSectionError = NoSectionError
        class MySettings(self._config.ConfigFileSettings):
            _config = Config(("foo", {"a": 1, "_b": self}))
        inst = MySettings()
        container = Container()
        try:
            inst._passConfig(container, "foo")
        except self._config.ConfigOptionUnknownError:
            # fine
            pass
        else:
            self.fail("Doesn't raise ConfigOptionUnknownError")

    def test16passConfig(self):
        """ ._passConfig raises exception on unknown options """
        class NoSectionError(Exception):
            pass
        self._mocked["ConfigParser"].NoSectionError = NoSectionError
        class MyContainer(Container):
            def _set_(this, name, value):
                raise AttributeError(name)
        class MySettings(self._config.ConfigFileSettings):
            _config = Config(("foo", {"a": 1, "b": self}))
        inst = MySettings()
        container = MyContainer()
        try:
            inst._passConfig(container, "foo")
        except self._config.ConfigOptionUnknownError:
            # fine
            pass
        else:
            self.fail("Doesn't raise ConfigOptionUnknownError")

    def test17passConfig(self):
        """ ._passConfig raises exception on unknown sections """
        class NoSectionError(Exception):
            pass
        self._mocked["ConfigParser"].NoSectionError = NoSectionError
        class MyConfig(Config):
            def options(this, *args, **kwargs):
                try:
                    Config.options(this, *args, **kwargs)
                except KeyError:
                    raise NoSectionError()
        class MySettings(self._config.ConfigFileSettings):
            _config = MyConfig(("foo", {"a": 1, "b": self}))
        inst = MySettings()
        container = Container()
        try:
            inst._passConfig(container, "bar")
        except self._config.ConfigSectionNotFoundError:
            # fine
            pass
        else:
            self.fail("Doesn't raise ConfigSectionNotFoundError")

    def test18loadConfig(self):
        """ ._loadConfig inits the config from file """
        result = []
        class ConfigParser(Config):
            def readfp(this, fp, name):
                result.append(("read", name))
                this.args = list(this.args)
                this.args.append(("general", {}))
        class MySettings(self._config.ConfigFileSettings):
            def _findConfig(this):
                def close():
                    pass
                return Container(name = "foo", close = close)
            def _applyCharset(this):
                result.append("charset")
            def _applyIncludes(this, filename):
                result.append(("includes", filename))
            def _applyMaps(this):
                result.append("maps")
            def _createConfigParser(this):
                return ConfigParser()

        inst = MySettings()
        inst._loadConfig()
        expected = [("read", "foo"), "charset", ("includes", "foo"), "maps"]
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test19loadConfig(self):
        """ ._loadConfig raises ConfigNotFoundError """
        result = []
        class ConfigParser(Config):
            def readfp(this, fp, name):
                result.append(("read", name))
                this.args = list(this.args)
                this.args.append(("general", {}))
        class MySettings(self._config.ConfigFileSettings):
            def _findConfig(this):
                def close():
                    raise IOError(0, "foo")
                return Container(name = "foo", close = close)
            def _applyCharset(this):
                result.append("charset")
            def _applyIncludes(this, filename):
                result.append(("includes", filename))
            def _applyMaps(this):
                result.append("maps")
            def _createConfigParser(this):
                return ConfigParser()

        inst = MySettings()
        try:
            inst._loadConfig()
        except self._config.ConfigNotFoundError:
            # fine
            pass
        else:
            self.fail("Doesn't raise ConfigNotFoundError")

    def test20loadConfig(self):
        """ ._loadConfig works without [general] """
        result = []
        class ConfigParser(Config):
            def readfp(this, fp, name):
                result.append(("read", name))
                this.args = list(this.args)
                this.args.append(("something", {}))
        class MySettings(self._config.ConfigFileSettings):
            def _findConfig(this):
                def close():
                    pass
                return Container(name = "foo", close = close)
            def _applyCharset(this):
                result.append("charset")
            def _applyIncludes(this, filename):
                result.append(("includes", filename))
            def _applyMaps(this):
                result.append("maps")
            def _createConfigParser(this):
                return ConfigParser()

        inst = MySettings()
        inst._loadConfig()
        expected = [("read", "foo"), "maps"]
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test21findConfig(self):
        """ ._findConfig returns <stdin> on "-" file name """
        import sys
        class MySettings(self._config.ConfigFileSettings):
            runtime = Container(config = "-")

        inst = MySettings()
        expected = sys.stdin
        result = inst._findConfig()
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test22findConfig(self):
        """ ._findConfig opens the file from runtime config """
        class MySettings(self._config.ConfigFileSettings):
            runtime = Container(config = "some filename")
        def filefunc(name, mode = "r"):
            if mode != "r":
                self.fail("Expected mode=r, got %r" % (mode,))
            return Container(name = name, mocked_file = True)

        inst = MySettings()
        expected = {'name': "some filename", 'mocked_file': True}
        result = inst._findConfig(_file = filefunc)._dict_
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test23findConfig(self):
        """ ._findConfig raises ConfigNotFoundError """
        class MySettings(self._config.ConfigFileSettings):
            runtime = Container(config = "some filename")
        def filefunc(name, mode = "r"):
            raise IOError(0, "foo")
        def utilfunc(name, encoding):
            return (name, encoding)
        self._mocked['svnmailer.util'].getSuitableFileName = utilfunc

        inst = MySettings()
        try:
            inst._findConfig(_file = filefunc)
        except self._config.ConfigNotFoundError:
            # fine.
            pass
        else:
            self.fail("Doesn't raise ConfigNotFoundError")

    def test24findConfig(self):
        """ ._findConfig tries the default files """
        import errno
        result = []
        class MySettings(self._config.ConfigFileSettings):
            runtime = Container(config = None)
            def _getDefaultConfigFiles(this):
                return ["foo", "bar"]
        def filefunc(name, mode = "r"):
            if mode != "r":
                self.fail("Expected mode=r, got %r" % (mode,))
            result.append(name)
            raise IOError(errno.ENOENT, "blub")

        inst = MySettings()
        try:
            inst._findConfig(_file = filefunc)
        except self._config.ConfigMissingError:
            # fine.
            pass
        else:
            self.fail("Doesn't raise ConfigMissingError")

        expected = ["foo", "bar"]
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test25findConfig(self):
        """ ._findConfig raises ConfigNotFoundError on default file """
        import errno
        class MySettings(self._config.ConfigFileSettings):
            runtime = Container(config = None)
            def _getDefaultConfigFiles(this):
                return ["foo", "bar"]
        def filefunc(name, mode = "r"):
            raise IOError(errno.EACCES, "blub")
        def utilfunc(name, encoding):
            return (name, encoding)
        self._mocked['svnmailer.util'].getSuitableFileName = utilfunc

        inst = MySettings()
        try:
            inst._findConfig(_file = filefunc)
        except self._config.ConfigNotFoundError:
            # fine.
            pass
        else:
            self.fail("Doesn't raise ConfigNotFoundError")

    def test26applyMaps(self):
        """ ._applyMaps doesn't choke without [maps] """
        class MySettings(self._config.ConfigFileSettings):
            _config = Config()
        inst = MySettings()
        inst._applyMaps()

        expected = []
        result = inst._config.sections_removed
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test27applyMaps(self):
        """ ._applyMaps may disable Mapping """
        class MySettings(self._config.ConfigFileSettings):
            MAPSECTION = None
            _config = Config((self._config.ConfigFileSettings.MAPSECTION, {}))
        inst = MySettings()
        inst._applyMaps()

        expected = []
        result = inst._config.sections_removed
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test28applyMaps(self):
        """ ._applyMaps creates mappings for all supplied options """
        class MySettings(self._config.ConfigFileSettings):
            _config = Config(
                ("maps", {"a": "[spam]", "b": "[eggs]", "c": "[spam]"})
            )
            def _getPlainMap(this, value):
                return ("mapper", value)
        inst = MySettings()
        inst._applyMaps()

        expected = {
            "a": ("mapper", "spam"), "b": ("mapper", "eggs"),
            "c": ("mapper", "spam")
        }
        result = inst._maps_
        self.failUnlessEqual(expected, result,
            "1st: Expected %r, got %r" % (expected, result)
        )

        expected = dict.fromkeys(["maps", "spam", "eggs"])
        result = dict.fromkeys(inst._config.sections_removed)
        self.failUnlessEqual(expected, result,
            "2nd: Expected %r, got %r" % (expected, result)
        )

    def test29applyMaps(self):
        """ ._applyMaps chokes on _ options """
        class MySettings(self._config.ConfigFileSettings):
            _config = Config(("maps", {"a": "[spam]", "_b": "[eggs]"}))
            def _getPlainMap(this, value):
                return ("mapper", value)
        inst = MySettings()
        try:
            inst._applyMaps()
        except self._config.ConfigOptionUnknownError:
            # fine
            pass
        else:
            self.fail("Doesn't raise ConfigOptionUnknownError")

    def test30applyMaps(self):
        """ ._applyMaps chokes on invalid mapping specs """
        tests = ("spam", "[spam", "spam]")
        class MySettings(self._config.ConfigFileSettings):
            def _getPlainMap(this, value):
                return ("mapper", value)
        inst = MySettings()
        for test in tests:
            inst._config = Config(("maps", {"a": test}))
            try:
                inst._applyMaps()
            except self._config.ConfigMappingSpecInvalidError:
                # fine
                pass
            else:
                self.fail(
                    "%r doesn't raise ConfigMappingSpecInvalidError" % (test,)
                )

    def test31getPlainMap(self):
        """ ._getPlainMap chokes on missing section """
        class NoSectionError(Exception):
            pass
        self._mocked["ConfigParser"].NoSectionError = NoSectionError
        class MyConfig(Config):
            def options(this, section):
                raise NoSectionError()
        class MySettings(self._config.ConfigFileSettings):
            _config = MyConfig()
        inst = MySettings()
        try:
            inst._getPlainMap("foo")
        except self._config.ConfigMappingSectionNotFoundError:
            # fine
            pass
        else:
            self.fail("Doesn't raise ConfigMappingSectionNotFoundError")

    def test32getPlainMap(self):
        """ ._getPlainMap returns a proper mapper function """
        class MySettings(self._config.ConfigFileSettings):
            _config = Config(("foo", {"a": 1, "b": self}))
        inst = MySettings()
        func = inst._getPlainMap("foo")

        expected = [1, self, "c"]
        result = [func("a"), func("b"), func("c")]
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test33applyIncludes(self):
        """ ._applyIncludes deals without include_config option """
        class NoOptionError(Exception):
            pass
        self._mocked["ConfigParser"].NoOptionError = NoOptionError
        class MyConfig(Config):
            def get(*args, **kwargs):
                try:
                    return Config.get(*args, **kwargs)
                except KeyError:
                    raise NoOptionError()
        class MySettings(self._config.ConfigFileSettings):
            _config = MyConfig(("general", {}))
        inst = MySettings()
        inst._applyIncludes("foo")

    def test34applyIncludes(self):
        """ ._applyIncludes deals with empty include_config option """
        class MySettings(self._config.ConfigFileSettings):
            _config = Config(("general", {'include_config': '  '}))
        inst = MySettings()
        inst._applyIncludes("foo")

        expected = [("general", "include_config")]
        result = inst._config.options_removed
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test35applyIncludes(self):
        """ ._applyIncludes processes all include files """
        import os
        result = []
        def filefunc(name, mode = "r"):
            if mode != "r":
                self.fail("Expected mode=r, got %r" % (mode,))
            def close():
                pass
            return Container(name = name, mocked_file = True, close = close)
        def splitfunc(tosplit):
            return tosplit.split()
        class _LocaleFile(object):
            def toLocale(this, name, *arg, **kwarg):
                return name
        self._mocked["svnmailer.util"].splitCommand = splitfunc
        self._mocked["svnmailer.util"].filename = _LocaleFile()

        class MyConfig(Config):
            def readfp(this, handle, name):
                result.append(name)
        class MySettings(self._config.ConfigFileSettings):
            _config = MyConfig(("general", {'include_config': 'a b c'}))
            _charset_ = None
            runtime = Container(path_encoding = None)
        inst = MySettings()
        inst._applyIncludes(
            os.path.abspath(os.path.join(os.path.sep, "foo", "bar.conf")),
            _file = filefunc
        )

        expected = [
            os.path.abspath(os.path.join(os.path.sep, "foo", name))
            for name in ('a', 'b', 'c')
        ]
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test36applyIncludes(self):
        """ ._applyIncludes raises ConfigNotFoundError """
        import os
        result = []
        def filefunc(name, mode = "r"):
            if mode != "r":
                self.fail("Expected mode=r, got %r" % (mode,))
            def close():
                pass
            return Container(name = name, mocked_file = True, close = close)
        def splitfunc(tosplit):
            return tosplit.split()
        class _LocaleFile(object):
            def toLocale(this, name, *arg, **kwarg):
                return name
        self._mocked["svnmailer.util"].splitCommand = splitfunc
        self._mocked["svnmailer.util"].filename = _LocaleFile()

        class MyConfig(Config):
            def readfp(this, handle, name):
                raise IOError(0, "foo")
        class MySettings(self._config.ConfigFileSettings):
            _config = MyConfig(("general", {'include_config': 'a b   c '}))
            _charset_ = None
            runtime = Container(path_encoding = None)
        inst = MySettings()

        try:
            inst._applyIncludes(
                os.path.join(os.path.sep, "foo", "bar.conf"), _file = filefunc
            )
        except self._config.ConfigNotFoundError:
            # fine
            pass
        else:
            self.fail("Doesn't raise ConfigNotFoundError")

    def test37applyCharset(self):
        """ ._applyCharset deals without config_charset option """
        class NoOptionError(Exception):
            pass
        self._mocked["ConfigParser"].NoOptionError = NoOptionError
        class MyConfig(Config):
            def get(*args, **kwargs):
                try:
                    return Config.get(*args, **kwargs)
                except KeyError:
                    raise NoOptionError()
        class MySettings(self._config.ConfigFileSettings):
            _config = MyConfig(("general", {}))
        inst = MySettings()
        inst._applyCharset()

    def test38applyCharset(self):
        """ ._applyCharset deals with empty config_charset option """
        class NoOptionError(Exception):
            pass
        self._mocked["ConfigParser"].NoOptionError = NoOptionError
        class MyConfig(Config):
            def get(*args, **kwargs):
                try:
                    return Config.get(*args, **kwargs)
                except KeyError:
                    raise NoOptionError()
        class MySettings(self._config.ConfigFileSettings):
            _config = MyConfig(("general", {'config_charset': ' \t'}))
            _charset_ = "foo"
        inst = MySettings()
        inst._applyCharset()

        expected = "foo"
        result = inst._charset_
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

        expected = [("general", "config_charset")]
        result = inst._config.options_removed
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test39applyCharset(self):
        """ ._applyCharset applies config_charset """
        class NoOptionError(Exception):
            pass
        self._mocked["ConfigParser"].NoOptionError = NoOptionError
        class MyConfig(Config):
            def get(*args, **kwargs):
                try:
                    return Config.get(*args, **kwargs)
                except KeyError:
                    raise NoOptionError()
        class MySettings(self._config.ConfigFileSettings):
            _config = MyConfig(("general", {'config_charset': 'foo'}))
            _charset_ = None
        inst = MySettings()
        inst._applyCharset()

        expected = "foo"
        result = inst._charset_
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

        expected = [("general", "config_charset")]
        result = inst._config.options_removed
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test40getDefaultConfigFiles(self):
        """ ._getDefaultConfigFiles returns localized file names """
        class MySettings(self._config.ConfigFileSettings):
            runtime = Container(path_encoding = None, repository = "blah")
        class _sys(object):
            argv = ["foo/\xa4uro"]
        _sys = _sys()
        class _os(object):
            import posixpath as path
        _os = _os()
        class _LocaleFile(object):
            def toLocale(this, name, *arg, **kwarg):
                return "to-" + name
            def fromLocale(this, name, *arg, **kwarg):
                return "from-" + name
        self._mocked["svnmailer.util"].filename = _LocaleFile()

        inst = MySettings()
        expected = [
            "blah/conf/mailer.conf", "from-foo/mailer.conf",
            "/etc/svn-mailer.conf"
        ]
        result = inst._getDefaultConfigFiles(_sys = _sys, _os = _os)
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

        _sys.argv[0] = u"foo/\u20ac"
        expected = [
            u"to-blah/conf/mailer.conf", u"to-from-foo/mailer.conf",
            u"to-/etc/svn-mailer.conf"
        ]
        result = inst._getDefaultConfigFiles(_sys = _sys, _os = _os)
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

_tests.append(ConfigFileSettingsTestCase)


#### Mocked stuff                                   ####
########################################################
class Container(object):
    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)
        self._dict_ = kwargs.copy()
    def _set_(self, name, value):
        setattr(self, name, value)
        self._dict_[name] = value

class Config(object):
    def __init__(self, *args):
        self.args = args
        self.sections_removed = []
        self.options_removed = []
    def sections(self):
        return [arg[0] for arg in self.args]
    def section(self, name):
        return dict(self.args)[name]
    def get(self, section, option, raw = False):
        if not raw:
            raise RuntimeError("raw is False!")
        return self.section(section)[option]
    def options(self, section):
        return self.section(section).keys()
    def remove_section(self, name):
        self.args = [arg for arg in self.args if arg[0] != name]
        self.sections_removed.append(name)
    def remove_option(self, section, option):
        self.args = list(self.args)
        for arg in self.args:
            if arg[0] == section:
                del arg[1][option]
        self.options_removed.append((section, option))
    def has_section(self, name):
        return dict(self.args).has_key(name)


#### Glue code                                      ####
########################################################
__all__.extend(_tests)

def getSuite():
    """ Returns the test suite for this module """
    return unittest.TestSuite([
        unittest.makeSuite(test) for test in _tests
    ])

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity = 2).run(getSuite())
